<?php

class Application_Model_VoteMapper
{
  protected $_dbTable;

  public function setDbTable($dbTable)
  {
    if (is_string($dbTable)) {
      $dbTable = new $dbTable();
    }
    if (!$dbTable instanceof Zend_Db_Table_Abstract) {
      throw new Exception('Invalid table data gateway provided');
    }
    $this->_dbTable = $dbTable;
    return $this;
  }

  public function getDbTable()
  {
    if (null === $this->_dbTable) {
      $this->setDbTable('Application_Model_DbTable_Vote');
    }
    return $this->_dbTable;
  }

  public function save(Application_Model_Vote $vote)
  {
    //Un utilisateur ne peut voter qu'une fois par jour pour un restaurant donné
    if (null == $this->getDbTable()->fetchRow(
      $this->getDbTable()->select()->where('votedate = ?',date('Y-m-d'))
       ->where('username = ?',$vote->getUsername())
       ->where('restaurantname = ?',$vote->getRestaurantname())
    )) {

      $data = array(
        'username' => $vote->getUsername(),
        'restaurantname' => $vote->getRestaurantname(),
        'votedate' => date('Y-m-d'),
      );

      $this->getDbTable()->insert($data);
    }
  }

  public function findByDate($date)
  {
    //récupération des votes pour une date données
    $resultSet = $this->getDbTable()->fetchAll(
      $this->getDbTable()->select()->where('votedate = ?',$date)->order('username ASC')
    );
    $entries = array();
    foreach ($resultSet as $row) {
      $entry = new Application_Model_Vote();
      $entry->setId($row->id)
        ->setUsername($row->username)
        ->setRestaurantname($row->restaurantname)
        ->setVotedate($row->votedate);
      $entries[] = $entry;
    }
    return $entries;
  }

  public function getSummaryArray($date)
  {
    //récupération d'un tableau résumant ne nombre de votes par restaurant
    $table = $this->getDbTable();
    $select = $table->select();

    $select->from($table,array('restaurantname','COUNT(*) as count'))
      ->where('votedate = ?',$date)
      ->group('restaurantname')
      ->order('count DESC');

    $resultSet = $table->fetchAll($select);

    $entries = array();
    foreach ($resultSet as $row) {
      $entry = Array(
        'restaurantname'=>$row->restaurantname,
        'count'=>$row->count
      );
      $entries[] = $entry;
    }
    return $entries;
  }
}
